#ifndef OBJ12_DB_API_H
#define OBJ12_DB_API_H

/* RDM 11.0.0.1000 */

#include "Db.h"
#include "Cursor.h"
#include "obj12_db.h"

namespace RDM_CPP {

class Cursor_office;

/*! \brief The obj12_db Db class
 *
 * This class provides the implementation for the obj12_db database
*/
class Db_obj12_db : public Db {
public:
    /*! \brief Open the obj12_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj12_db object
     */
    static Db_obj12_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj12_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj12_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj12_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj12_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj12_db object
     */
    static Db_obj12_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj12_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj12_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj12_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj12_db Default Constructor
    Db_obj12_db (void) throw ();
    /*! \brief Db_obj12_db Constructor
     *
     * \param db A Db object used to create the Db_obj12_db
     */
    Db_obj12_db (const Db& db) throw ();
    //! Db_obj12_db Copy Constructor
    Db_obj12_db (const Db_obj12_db&) throw ();

    //! Db_obj12_db assignment operator
    Db_obj12_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj12_db assignment operator
    Db_obj12_db& operator = (const Db_obj12_db&) throw ();

    /*! \brief Db_obj12_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj12_db ();

    /*! \brief Create a new office record
     *
     * \returns A singleton Cursor_office object with default field values
     *
     * Create a new office record using default values
     */
    Cursor_office New_office_record (void) const throw (rdm_exception&);

    /*! \brief Create a new office record
     *
     * \param fields The field values for the new office record
     * \returns A singleton Cursor_office object with the specified field values
     *
     * Create a new office record using the specified values
     */
    Cursor_office New_office_recordWithFieldValues (struct office *fields) const throw (rdm_exception&);

    /*! \brief Obtain office cursor position at the first office record
     * \returns office cursor positioned at the first office record
     *
     * Create a new office cursor for doing a table scan of the office records.
     * The cursor is positioned at the first office record
     */
    Cursor_office Get_office_records (void) const throw (rdm_exception&);

    /*! \brief Obtain a office cursor based on the name key
     * \returns office cursor positioned at the first office name key value
     *
     * Create a new office cursor based on the name key and position the cursor
     * at the first record
     */
    Cursor_office Get_office_recordsBy_name () const throw (rdm_exception&);

private:
    /*! \brief Db_obj12_db reference operator */
    Db_obj12_db* operator & (void);
    /*! \brief Db_obj12_db constructor */
    Db_obj12_db (IDb *idb);
    /*! \brief Get the Db_obj12_db dbType
     * \return The Db_obj12_db dbType
     */
    static dbType Type (void);
};

/*! \brief The office Cursor class
 *
 * This class provides the implementation for the office Cursor
*/
class Cursor_office : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_office objects
     */
    static recordType Type (void);

    /*! Cursor_office Default Constructor */
    Cursor_office (void) throw ();
    /*! \brief Cursor_office Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_office object
     */
    Cursor_office (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_office Copy Constructor */
    Cursor_office (const Cursor_office&) throw ();

    /*! \brief Cursor_office assignment operator */
    Cursor_office& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_office assignment operator */
    Cursor_office& operator = (const Cursor_office& cursor) throw ();

    /*! \brief Create a new office cursor doing name key navigation.
     *
     * \return office cursor ordered by the name key
     *
     *  Create a new office cursor using name key navigation and position the
     *  new cursor at our current position
     */
    Cursor_office GetRecordsBy_name () const throw (rdm_exception&);

    /*! \brief Position the office cursor to the name key value
     *
     * \param name The name value to position the cursor at
     *
     */
    void KeyFindBy_name (char name_val[32]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct office *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct office *fields) const throw (rdm_exception&);

    /*! \brief Set the name field value for the current record
     * \param name The field value that will be written to the record at the current cursor position
     */
    void Set_name (const char name_val[32]) const throw (rdm_exception&);

    /*! \brief Get the name field value for the current record
     * \param name A pointer where the field value from the current cursor position will be written
     */
    void Get_name (char name_val[32]) const throw (rdm_exception&);

    /*! \brief Cursor_office Destructor
     *
     * Release the cursor
     */
    ~Cursor_office ();

private:
    friend class Db_obj12_db;
};

}
#endif    /* OBJ12_DB_API_H */
